#include <cstdio>
#include <map>
#include <string>
#include <utility>

using namespace std;

typedef map<unsigned, string> freqmap;
typedef pair<unsigned, unsigned> modedef;
typedef map<modedef, freqmap> modemap;

static modemap modes;

static void ReadModeList()
{
    FILE *fp = fopen("/etc/fb.modes", "rt");
    if(!fp) return;
    
    for(;;)
    {
        char Buf[1024];
        if(!fgets(Buf, sizeof Buf, fp))break;
        if(strncmp(Buf, "mode ", 5)) continue;
        
        const char *p1 = strchr(Buf, '"');
        if(!p1)continue;
        const char *p2 = strchr(p1+1, '"');
        if(!p2)continue;
        
        string modestr(p1+1, p2-p1-1);
        
        unsigned x, y, ref;
        if(sscanf(modestr.c_str(), "%ux%u-%u", &x, &y, &ref) != 3) continue;
        
        //printf("%u x %u @ %u Hz\n", x, y, ref);
        
        modes[make_pair(x,y)][ref] = modestr;
    }
    
    fclose(fp);
}

static void FindModes(unsigned minx, unsigned miny)
{
    modemap::const_iterator i;
    bool found = false;
    modedef best;
    unsigned bestspace = 0x7FFFFFFF;
    for(i = modes.begin(); i != modes.end(); ++i)
    {
        modedef def = i->first;
        
        unsigned x = def.first;
        unsigned y = def.second;
        
        if(x < minx || y < miny) continue;
        
        found = true;
        
        unsigned space = (x - minx) * (y - miny);
        if(space < bestspace)
        {
            best = def;
            bestspace = space;
            found = true;
        }
    }
    if(!found) return;
    
    const freqmap &freqs = modes[best];
    freqmap::const_iterator j;
    for(j = freqs.end(); j-- != freqs.begin(); )
    {
        puts(j->second.c_str());
        return;
        
        //printf("%u Hz: %s\n", j->first, j->second.c_str());
    }
}

int main(int argc, const char *const *argv)
{
    if(argc != 3)
    {
        fprintf(stderr,
        	"mplaymode v"VERSION" Copyright (C) 1992,2003 Bisqwit (http://bisqwit.iki.fi/)\n"
        	"Finds the most optimal resolution (uses /etc/fb.modes).\n"
        	"Usage: mplaymode <xres> <yres>\n");
        return 0;
    }
    ReadModeList();
    FindModes(atoi(argv[1]), atoi(argv[2]));
    return 0;
}
